/*
 * Copyright (C) 2013, 2014 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 */

import QtQuick 2.0
import Ubuntu.Layouts 0.1
import Ubuntu.Components 0.1
import "../common/ClockUtils.js" as Utils
import "../common"

Page {
    id: alarmPage
    objectName: "alarmPage"

    property Flickable alarmPageFlickable

    readonly property real alarmHeaderHeight: units.gu(9.5)
    property string currentTime
    property int enabledAlarmCount: 0
    property var nextAlarm
    property real tempPosition

    flickable: !wideAspect ? alarmPageFlickable : null

    Component.onCompleted: {
        Utils.log("AlarmPage loaded")
        alarmModel.count > 0 ? get_next_active_alarm() : Utils.log("No Alarms Present")
        currentTime = Utils.convertTime(new Date().getHours(), new Date().getMinutes(), new Date().getUTCSeconds(), appSetting.contents.timeFormat)
    }
    Component.onDestruction: Utils.log("AlarmPage unloaded")

    actions: [
        Action {
            id: addAlarmAction
            iconSource: Qt.resolvedUrl("../images/add_icon.png")
            text: i18n.tr("Add alarm")
            description: i18n.tr("Add a new alarm")
            keywords: i18n.tr("Add;Alarm;Alert;New;Tone;Bell")
            onTriggered: pagestack.push(Qt.resolvedUrl("AddAlarmPage.qml"), {"isNewAlarm": true})
        }
    ]

    function onTimerUpdate(now) {
        currentTime = Utils.convertTime(new Date().getHours(), new Date().getMinutes(), new Date().getUTCSeconds(), appSetting.contents.timeFormat)
    }

    function get_next_active_alarm()
    {
        enabledAlarmCount = 0
        var tempAlarm;

        for(var i=0; i<alarmModel.count; i++) {
            tempAlarm = alarmModel.get(i)
            if (tempAlarm.enabled) {
                enabledAlarmCount += 1
                nextAlarm = alarmModel.get(i)
                break;
            }
        }

        if (enabledAlarmCount !== 0) {
            for (var i=0; i<alarmModel.count; i++) {
                tempAlarm = alarmModel.get(i)
                if (tempAlarm.enabled)
                    if (tempAlarm.date < nextAlarm.date)
                        nextAlarm = alarmModel.get(i)
            }
            nameAlarm.text = nextAlarm.message
            alarmOccurrence.text = format_day_string(nextAlarm.daysOfWeek, nextAlarm.type)
            alarmFace.hours = Utils.circlePositiontoHours(Qt.formatTime(nextAlarm.date, "hh AP").split(' ')[0], "12-hour")
            alarmFace.minutes = Qt.formatTime(nextAlarm.date, "mm")
            alarmFace.innerLabel.text = Utils.convertTime(Qt.formatTime(nextAlarm.date, "hh"), Qt.formatTime(nextAlarm.date, "mm"), 0, alarmFace.timeFormat)
        }
        else {
            nameAlarm.text = "";
            alarmOccurrence.text = "";
            alarmFace.hours = 0;
            alarmFace.minutes = 0;
            alarmFace.innerLabel.text = "00:00"
        }
    }

    function format_day_string(value, type) {
        var occurs = get_day(value, type);
        var WeekDay = 0;
        for (var i = 0; i < Qt.locale().weekDays.length; ++i) {
            switch (Qt.locale().weekDays[i]) {
            case Qt.Monday:
                WeekDay |= Alarm.Monday
                break;
            case Qt.Tuesday:
                WeekDay |= Alarm.Tuesday
                break;
            case Qt.Wednesday:
                WeekDay |= Alarm.Wednesday
                break;
            case Qt.Thursday:
                WeekDay |= Alarm.Thursday
                break;
            case Qt.Friday:
                WeekDay |= Alarm.Friday
                break;
            case Qt.Saturday:
                WeekDay |= Alarm.Saturday
                break;
            case Qt.Sunday:
                WeekDay |= Alarm.Sunday
                break;
            }
        }

        if (value == WeekDay) {
            return i18n.tr("Every weekday")
        }
        else if (value == Alarm.Daily) {
            return i18n.tr("Daily")
        }
        else {
            if (type === Alarm.Repeating)
                return i18n.tr("Every ") + occurs
            else
                return i18n.tr("Once on ")  + occurs
        }
    }

    function get_day(value, type) {
        var occurs = [];
        if (value & Alarm.Monday) occurs.push(Qt.locale().standaloneDayName(1, Locale.ShortFormat));
        if (value & Alarm.Tuesday) occurs.push(Qt.locale().standaloneDayName(2, Locale.ShortFormat));
        if (value & Alarm.Wednesday) occurs.push(Qt.locale().standaloneDayName(3, Locale.ShortFormat));
        if (value & Alarm.Thursday) occurs.push(Qt.locale().standaloneDayName(4, Locale.ShortFormat));
        if (value & Alarm.Friday) occurs.push(Qt.locale().standaloneDayName(5, Locale.ShortFormat));
        if (value & Alarm.Saturday) occurs.push(Qt.locale().standaloneDayName(6, Locale.ShortFormat));
        if (value & Alarm.Sunday) occurs.push(Qt.locale().standaloneDayName(7, Locale.ShortFormat));
        return occurs;
    }

    // Layouts rules for phone and tablet interfaces
    Layouts {
        anchors.fill: parent
        layouts: [
            // Tablet/Desktop Layout rules
            ConditionalLayout {
                name: "tablet"
                when: wideAspect
                Row {
                    anchors.fill: parent
                    // Container to hold the alarm face, alarm name and occurrence
                    Item {
                        id: alarmFaceTabletLayoutContainer
                        width: alarmModel.count !== 0 ? alarmPage.width - sidebar.width : alarmPage.width
                        anchors {
                            top: parent.top
                            bottom: parent.bottom
                        }

                        Behavior on width {
                            UbuntuNumberAnimation {}
                        }

                        ItemLayout {
                            id: alarmFaceTabletItem
                            item: "alarmFace"
                            width: alarmFace.width
                            height: alarmFace.height
                            anchors {
                                centerIn: parent
                                verticalCenterOffset: - nameAlarm.height - alarmOccurrence.height - units.gu(2)
                            }
                        }

                        ItemLayout {
                            id: nameAlarmTabletItem
                            item: "nameAlarm"
                            width: nameAlarm.contentWidth
                            height: nameAlarm.contentHeight
                            anchors {
                                horizontalCenter: parent.horizontalCenter
                                top: alarmFaceTabletItem.bottom
                                topMargin: units.gu(5)
                            }
                        }

                        ItemLayout {
                            id: alarmOccurrenceTabletItem
                            item: "alarmOccurrence"
                            width: alarmOccurrence.contentWidth
                            height: alarmOccurrence.contentHeight
                            anchors {
                                horizontalCenter: parent.horizontalCenter
                                top: nameAlarmTabletItem.bottom
                            }
                        }
                    }

                    // Sidebar to hold the world clock column in the tablet layout
                    Sidebar {
                        id: sidebar
                        mode: "right"
                        visible: alarmModel.count !== 0

                        anchors {
                            top: parent.top
                            bottom: parent.bottom
                        }

                        ItemLayout {
                            item: "listAlarm"
                            width: parent.width
                        }
                    }
                }
            }
        ]

        Flickable {
            id: alarmAnimationContainer
            objectName: "animationContainer"

            Component.onCompleted: alarmPageFlickable = alarmAnimationContainer

            anchors.fill: parent
            contentWidth: alarmPage.width
            contentHeight: alarmFacePhoneLayoutContainer.height + listAlarm.height

            Item {
                id: alarmFacePhoneLayoutContainer

                width: parent.width
                height: alarmPage.height - alarmPage.alarmHeaderHeight - listAlarm.headerHeight - listAlarm.dividerHeight

                AlarmFace {
                    id: alarmFace
                    Layouts.item: "alarmFace"

                    anchors {
                        centerIn: parent
                        verticalCenterOffset: - nameAlarm.height - alarmOccurrence.height
                    }

                    states: [
                        State {
                            name: "NOACTIVEALARMS"
                            when: enabledAlarmCount === 0 && alarmModel.count !== 0
                            PropertyChanges { target: alarmFace.nextAlarmText; text: i18n.tr("No Active Alarms"); fontSize: "large"; anchors.verticalCenterOffset: 0 }
                            PropertyChanges { target: alarmFace.innerLabel; visible: false }
                            PropertyChanges { target: alarmFace.currentTimeLabel; visible: false }
                        }
                    ]

                    currentTimeLabel.text: i18n.tr("Now") + " " + currentTime
                    showSetupMessage: alarmModel.count === 0
                    onTimeFormatChanged: get_next_active_alarm()
                    onPressAndHold: pagestack.push(Qt.resolvedUrl("AddAlarmPage.qml"), {"isNewAlarm": true})
                }

                // Label to show the name of the next active alarm
                Label {
                    id: nameAlarm
                    Layouts.item: "nameAlarm"

                    fontSize: "large"
                    font.bold: true
                    width: contentWidth
                    height: contentHeight
                    anchors {
                        horizontalCenter: parent.horizontalCenter
                        top: alarmFace.bottom
                        topMargin: units.gu(5)
                    }
                }

                Label {
                    id: alarmOccurrence
                    Layouts.item: "alarmOccurrence"
                    width: contentWidth
                    height: contentHeight
                    anchors {
                        horizontalCenter: parent.horizontalCenter
                        top: nameAlarm.bottom
                    }
                }
            }

            // Column element to hold the saved alarms
            AlarmList {
                id: listAlarm
                Layouts.item: "listAlarm"
                model: alarmModel
                height: childrenRect.height
                width: parent.width
                anchors.top: alarmFacePhoneLayoutContainer.bottom
                maxItemsVisible: !wideAspect ? 5 : parseInt((alarmPage.height - headerHeight)/units.gu(6))
            }
        }
    }

    tools: ToolbarItems {
        id: toolbarAlarm

        ToolbarButton {
            id: settings
            action: appSettingsAction
        }

        ToolbarButton {
            id: addPresetToolbarButton
            objectName: "addAlarmButton"
            action: addAlarmAction
        }
    }
}
