/*
 * Copyright (C) 2013, 2014 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Alessandro Pozzi <signor.hyde@gmail.com>
 *              Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 *              Renato Araujo Oliveira Filho <renato.filho@canonical.com>
 */

import QtQuick 2.0
import Ubuntu.Layouts 0.1
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import "../common/ClockUtils.js" as Utils
import "StopwatchSupport.js" as SScript
import "../common"

Page {
    id: stopwatchPage

    property Flickable stopwatchPageFlickable

    readonly property real stopwatchHeaderHeight: units.gu(9.5)
    property alias lapStart: laps.lapStart
    property bool isRunning: analogStopwatch.timerStatus &&
                             analogStopwatch.hours !== 0 || analogStopwatch.minutes !== 0 || analogStopwatch.seconds !== 0 || analogStopwatch.milliseconds !== 0

    flickable: !wideAspect ? stopwatchPageFlickable : null
    Component.onCompleted: Utils.log("StopwatchPage loaded");
    Component.onDestruction: Utils.log("Stopwatch unloaded");

    // Function to create a lap
    function create_lap() {
        if(analogStopwatch.timerStatus === true) {
            laps.push();
            analogStopwatch.blipLocation = analogStopwatch.seconds
            analogStopwatch.blipVisible = true
        }
    }

    // Function to start/stop a stopwatch
    function start_stop_stopwatch() {
        analogStopwatch.stopwatchAnimationHint.start()
        if(analogStopwatch.timerStatus == false) {
            analogStopwatch.stopwatchHintText = i18n.tr("Stop")
            analogStopwatch.start()
            stopWatchLabelUpdate.start()
            if (analogStopwatch.flag) {
                analogStopwatch.startTime = new Date().getTime();
                analogStopwatch.flag = false;
            }
            else {
                analogStopwatch.delay = Math.abs(new Date().getTime() - analogStopwatch.stopTime);
                analogStopwatch.startTime = analogStopwatch.startTime + analogStopwatch.delay;
            }
        } else {
            analogStopwatch.stopwatchHintText = i18n.tr("Start")
            analogStopwatch.stop()
            stopWatchLabelUpdate.stop()
            analogStopwatch.stopTime = new Date().getTime();
        }
    }

    // Function to reset stopwatch
    function reset_stopwatch() {
        analogStopwatch.stopwatchHintText = i18n.tr("Start")
        analogStopwatch.stopwatchAnimationHint.start()
        analogStopwatch.stop()
        analogStopwatch.reset();
        analogStopwatch.innerLabel.text = SScript.timeToString(0, 0, 0, 0);
        lapStart = 0;
        laps.clear();
    }

    // Timer which runs every 100 microseconds to update the stopwatch label
    Timer {
        id: stopWatchLabelUpdate;

        interval: 100;
        repeat: true;
        onTriggered: analogStopwatch.innerLabel.text = SScript.timeToString(analogStopwatch.hours, analogStopwatch.minutes, analogStopwatch.seconds, analogStopwatch.milliseconds)
    }

    ListModel {
        id: laps

        property int lapStart: 0;

        function push() {
            insert(0, {"time" : (analogStopwatch.totalTime - lapStart),
                       "label": SScript.currentLapLabel(analogStopwatch.totalTime, lapStart)})
            lapStart = analogStopwatch.totalTime
        }
    }

    // Layout rules for phone and tablet interfaces
    Layouts {
        anchors.fill: parent
        layouts: [
            // Tablet/Desktop Layout rules
            ConditionalLayout {
                name: "tablet"
                when: wideAspect
                Row {
                    anchors.fill: parent
                    Item {
                        id: stopwatchFaceTabletLayoutContainer
                        width: laps.count !== 0 ? stopwatchPage.width - sidebar.width : stopwatchPage.width

                        Behavior on width {
                            UbuntuNumberAnimation {}
                        }

                        anchors {
                            top: parent.top
                            bottom: parent.bottom
                        }

                        ItemLayout {
                            id: stopwatchFaceTabletItem
                            item: "stopwatchFace"
                            width: analogStopwatch.width
                            height: analogStopwatch.height
                            anchors {
                                centerIn: parent
                                verticalCenterOffset: - lapButton.height - units.gu(2)
                            }
                        }

                        ItemLayout {
                            id: resetButtonTabletItem
                            item: "resetButton"
                            width: resetButton.width
                            height: resetButton.height
                            anchors {
                                right: stopwatchFaceTabletItem.left
                                rightMargin: units.gu(-1)
                                top: stopwatchFaceTabletItem.bottom
                                topMargin: units.gu(5)
                            }
                        }

                        ItemLayout {
                            id: lapButtonTabletItem
                            item: "lapButton"
                            width: lapButton.width
                            height: lapButton.height
                            anchors {
                                left: stopwatchFaceTabletItem.right
                                leftMargin: units.gu(-1)
                                top: stopwatchFaceTabletItem.bottom
                                topMargin: units.gu(5)
                            }
                        }

                        ItemLayout {
                            id: currentLapColumnTabletItem
                            item: "currentLapColumn"
                            height: currentLapColumn.height
                            anchors {
                                horizontalCenter: stopwatchFaceTabletItem.horizontalCenter
                                verticalCenter: lapButtonTabletItem.verticalCenter
                            }
                        }
                    }

                    // Sidebar to hold the world clock column in the tablet layout
                    Sidebar {
                        id: sidebar
                        mode: "right"
                        visible: laps.count !== 0

                        anchors {
                            top: parent.top
                            bottom: parent.bottom
                        }

                        ItemLayout {
                            item: "listLap"
                            width: parent.width
                        }
                    }
                }
            }
        ]

        Flickable {
            id: stopwatchAnimationContainer
            objectName: "animationContainer"

            Component.onCompleted: stopwatchPageFlickable = stopwatchAnimationContainer

            anchors.fill: parent
            contentWidth: stopwatchPage.width
            contentHeight: stopwatchFacePhoneLayoutContainer.height + listLap.height

            // Container to hold the stopwatch face, lap and reset buttons
            Item {
                id: stopwatchFacePhoneLayoutContainer

                width: parent.width
                height: stopwatchPage.height - stopwatchPage.stopwatchHeaderHeight - listLap.headerHeight - listLap.dividerHeight

                // Component which draws the analog stopwatch face, controls the timer and calculates the time.
                StopwatchFace {
                    id: analogStopwatch
                    objectName: "stopwatchFace"
                    Layouts.item: "stopwatchFace"

                    anchors {
                        centerIn: parent
                        verticalCenterOffset: - lapButton.height
                    }

                    visible: true
                    innerLabel.text: SScript.timeToString(hours, minutes, seconds, milliseconds)
                    onClicked: start_stop_stopwatch()
                }

                ImageButton {
                    id: resetButton
                    objectName: "resetButton"
                    Layouts.item: "resetButton"

                    anchors {
                        right: analogStopwatch.left
                        rightMargin: units.gu(-1)
                        top: analogStopwatch.bottom
                        topMargin: units.gu(5)
                    }

                    buttonImage: Qt.resolvedUrl("../images/reset_icon.svg");
                    buttonLabel: ""
                    onClicked: reset_stopwatch()
                }

                Column {
                    id: currentLapColumn
                    Layouts.item: "currentLapColumn"

                    visible: analogStopwatch.hours !== 0 || analogStopwatch.minutes !== 0 || analogStopwatch.seconds !== 0 || analogStopwatch.milliseconds !== 0
                    height: childrenRect.height
                    anchors {
                        horizontalCenter: analogStopwatch.horizontalCenter
                        verticalCenter: lapButton.verticalCenter
                    }

                    Label {
                        fontSize: "large"
                        anchors.horizontalCenter: parent.horizontalCenter
                        text: SScript.currentLapLabel(analogStopwatch.totalTime, laps.lapStart);
                    }

                    Label {
                        text: i18n.tr("Current")
                        anchors.horizontalCenter: parent.horizontalCenter
                    }
                }

                ImageButton {
                    id: lapButton
                    objectName: "lapButton"
                    Layouts.item: "lapButton"

                    anchors {
                        left: analogStopwatch.right
                        leftMargin: units.gu(-1)
                        top: analogStopwatch.bottom
                        topMargin: units.gu(5)
                    }

                    buttonImage: Qt.resolvedUrl("../images/lap_icon.png");
                    buttonLabel: ""
                    mirrorLayout: true;
                    onClicked: create_lap()
                }
            }

            // ListView to display the current lap and recorded lap information.
            LapList {
                id: listLap
                Layouts.item: "listLap"
                lapModel: laps
                width: parent.width
                height: childrenRect.height
                anchors.top: stopwatchFacePhoneLayoutContainer.bottom
                maxItemsVisible: !wideAspect ? 5 : parseInt((stopwatchPage.height - headerHeight)/units.gu(6))

            }
        }
    }
}
