# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013, 2014 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authored by: Nekhelesh Ramananthan <krnekhelesh@gmail.com>
#              Nicholas Skaggs <nicholas.skaggs@canonical.com>

import copy
import logging
from time import sleep

from autopilot.introspection import dbus
from autopilot import logging as autopilot_logging

from ubuntuuitoolkit import emulators as toolkit_emulators

logger = logging.getLogger(__name__)


class ClockEmulatorException(toolkit_emulators.ToolkitEmulatorException):
    """Exception raised when there is an error with the emulator."""


class MainView(toolkit_emulators.MainView):

    @autopilot_logging.log_action(logger.info)
    def open_timer(self):
        """Open the Timer tab.

        :return: the Timer page.

        """
        self.switch_to_tab('TimerTab')
        return self.wait_select_single(TimerPage)

    @autopilot_logging.log_action(logger.info)
    def open_alarm(self):
        """Open the Alarm Tab.

        :return: the Alarm page.

        """
        self.switch_to_tab('AlarmTab')
        return self.wait_select_single(AlarmPage)

    @autopilot_logging.log_action(logger.info)
    def open_stopwatch(self):
        """Open the Stopwatch Tab.

        :return the Stopwatch page.

        """
        self.switch_to_tab('StopwatchTab')
        return self.wait_select_single(StopwatchPage)

    @autopilot_logging.log_action(logger.info)
    def open_clock(self):
        """Open the Clock Tab.

        :return the Clock Page

        """
        return self.wait_select_single(ClockPage)


class LabelDots(toolkit_emulators.TextField):
    """Autopilot helper for the LabelDots component."""


class Page(toolkit_emulators.UbuntuUIToolkitEmulatorBase):
    """Autopilot helper for Pages."""

    def __init__(self, *args):
        super(Page, self).__init__(*args)
        # XXX we need a better way to keep reference to the main view.
        # --elopio - 2014-01-31
        self.main_view = self.get_root_instance().select_single(MainView)

    def drag_page_up(self):
        """Drag the given page up."""
        self._drag_page(direction='up')

    def drag_page_down(self):
        """Drag the given page down."""
        self._drag_page(direction='down')

    def _drag_page(self, direction):
        """Function to drag the page up/down."""
        self._wait_to_stop_moving()

        x, y, w, h = self.globalRect
        start_x = stop_x = x + (w / 2)
        start_y = y + (h / 2)

        if direction == "down":
            stop_y = start_y + h / 3
            self.pointing_device.drag(start_x, start_y, stop_x, stop_y)
        else:
            stop_y = start_y - h / 3
            self.pointing_device.drag(start_x, start_y, stop_x, stop_y)

        self._wait_to_stop_moving()

    def _wait_to_stop_moving(self):
        self.select_single(
            'QQuickFlickable',
            objectName='animationContainer').moving.wait_for(False)


class ClockPage(Page):
    """Autopilot helper for the Clock page."""

    def is_12_hour_time(self):
        """Check if AM or PM is shown in the clock time text"""
        clock_time = self.wait_select_single('Label', objectName='label').text
        if 'AM' in clock_time or 'PM' in clock_time:
            return True
        else:
            return False

    def get_num_of_saved_cities(self):
        """Return the number of saved world cities"""
        return int(self._get_saved_world_city_list().count)

    def _get_saved_world_city_list(self):
        """Return the saved world city listview object"""
        return self.wait_select_single(
            toolkit_emulators.QQuickListView, objectName='listWorldClocks')

    @autopilot_logging.log_action(logger.info)
    def add_world_city(self, index):
        """Add world city

        :param index: world city number

        """
        world_clock_page = self._click_add_city_button()
        world_clock_page.click_world_city(index=index)
        try:
            self._confirm_world_city_addition(index)
        except AssertionError:
            raise ClockEmulatorException('Error adding world city.')

    def _click_add_city_button(self):
        """Click the add city toolbar button"""
        toolbar = self.main_view.open_toolbar()
        toolbar.click_button('addCityAction')
        return self.main_view.wait_select_single(WorldClock)

    def _confirm_world_city_addition(self, index):
        """Confirm existence of saved world city

        :param index: world city number

        """
        self.wait_select_single(
            'Standard', objectName='savedworldcity{}'.format(index))

    @autopilot_logging.log_action(logger.info)
    def delete_world_city(self, index):
        """Delete world city

        :param index: world city number

        """
        old_world_city_count = self.get_num_of_saved_cities()
        world_city = self._get_saved_world_city(index=index)
        if not self.main_view.wideAspect:
            self.drag_page_up()
        world_city.swipe_to_delete()
        world_city.confirm_removal()
        try:
            self._confirm_world_city_deletion(old_count=old_world_city_count)
        except AssertionError:
            raise ClockEmulatorException('Error deleting world city.')

    def _get_saved_world_city(self, index):
        """Return saved world city object"""
        return self.wait_select_single(
            'Standard', objectName='savedworldcity{}'.format(index))

    def _confirm_world_city_deletion(self, old_count):
        """Confirm the removal of saved world city

        :param old_count: original of number of saved world cities

        """
        self._get_saved_world_city_list().count.wait_for(old_count - 1)

    @autopilot_logging.log_action(logger.info)
    def show_seconds_hand(self):
        """Enable show seconds hand settings option"""
        settings_page = self.go_to_settings_page()
        settings_page.enable_show_seconds_option()
        self._click_back_button()

    @autopilot_logging.log_action(logger.info)
    def go_to_settings_page(self):
        """Open the settings page by clicking the settings toolbar button"""
        toolbar = self.main_view.open_toolbar()
        toolbar.click_button('showSettingAction')
        return SettingsPage.select(self.main_view)

    def _click_back_button(self):
        """Click the back toolbar button"""
        toolbar = self.main_view.open_toolbar()
        toolbar.click_back_button()

    def is_seconds_hand_visible(self):
        """Return the clock seconds hand"""
        try:
            seconds_hand = self.wait_select_single(
                'DialerHand', objectName='secondHand')
            return seconds_hand.visible
        except dbus.StateNotFoundError:
            return False

    @autopilot_logging.log_action(logger.info)
    def enable_12_hour_time_format(self):
        """Set 12 hour time format"""
        settings_page = self.go_to_settings_page()
        settings_page.change_time_format_option('12-hour')
        self._click_back_button()

    @autopilot_logging.log_action(logger.info)
    def enable_24_hour_time_format(self):
        """Set 24 hour time format"""
        settings_page = self.go_to_settings_page()
        settings_page.change_time_format_option('24-hour')
        self._click_back_button()


class WorldClock(toolkit_emulators.UbuntuUIToolkitEmulatorBase):
    """Autopilot helper for the World Clock Page"""

    def click_world_city(self, index):
        """Click world city in the world city list

        :param index: world city number

        """
        city = self.wait_select_single(
            toolkit_emulators.Base, objectName='worldcity{}'.format(index))
        self.pointing_device.click_object(city)


class SettingsPage(object):
    """Autopilot helper for the Settings page."""

    def __init__(self, proxy_object):
        super(SettingsPage, self).__init__()
        self.proxy_object = proxy_object

    @classmethod
    def select(cls, main_view):
        proxy_object = main_view.wait_select_single(
            Page, objectName='settingsPage')
        proxy_object.visible.wait_for(True)
        return cls(proxy_object)

    @autopilot_logging.log_action(logger.info)
    def enable_show_seconds_option(self):
        """Enable the show seconds option"""
        show_seconds_checkbox = self.proxy_object.wait_select_single(
            toolkit_emulators.CheckBox, objectName='showSecondsCheckbox')
        show_seconds_checkbox.check()

    @autopilot_logging.log_action(logger.info)
    def change_time_format_option(self, format):
        """Set time format settings option

        :param time_format: time format index in the option selector

        """
        time_format_selector = self.proxy_object.wait_select_single(
            'OptionSelector', objectName='timeFormatSelector')

        # Wait for the option selector to expand
        # FIXME: Replace this when the ui toolkit provides option selector
        # emulator functions. http://pad.lv/1277059 -- nik90 2014-03-06
        if not time_format_selector.currentlyExpanded:
            # Click on time format option selector to expand it
            self.proxy_object.pointing_device.click_object(time_format_selector)
            sleep(2)
            time_format_selector.currentlyExpanded.wait_for(True)

        # Grab the labels after the option selector has expanded
        time_format_object = time_format_selector.select_single(
            'StyledItem', objectName='listContainer')
        time_formats = time_format_object.select_many('Label', visible=True)

        for index, format_element in enumerate(time_formats):
            if format_element.text == format:
                format_index = index
                break
        else:
            raise ClockEmulatorException(
                'Unknown settings option: {}'.format(format))

        # Click on the time format option
        self.proxy_object.pointing_device.click_object(
            time_formats[format_index])


class StopwatchPage(Page):
    """Autopilot helper for the Stopwatch page."""

    def get_stopwatch_time(self):
        """Return the stopwatch time"""
        return self.wait_select_single('Label', objectName='label').text

    def get_num_of_laps(self):
        """Return the number of laps in the stopwatch page."""
        return int(self.wait_select_single('QQuickListView').count)

    def _get_stopwatch_lap_list_element(self, index):
        """Return a lap list item in the stopwatch page.

        :param index: stopwatch lap number

        """
        return self.wait_select_single(
            'Standard', objectName='laps{}'.format(index))

    @autopilot_logging.log_action(logger.info)
    def start_stopwatch(self):
        """Start the stopwatch"""
        self._click_start_stop_stopwatch_button()

    @autopilot_logging.log_action(logger.info)
    def stop_stopwatch(self):
        """Stop the stopwatch"""
        self._click_start_stop_stopwatch_button()

    def _click_start_stop_stopwatch_button(self):
        """Click the start/stop stopwatch button"""
        start_stop_button = self.wait_select_single(
            'StopwatchFace', objectName='stopwatchFace')
        self.pointing_device.click_object(start_stop_button)

    @autopilot_logging.log_action(logger.info)
    def reset_stopwatch(self):
        """Reset the stopwatch"""
        self._click_reset_stopwatch_button()

    def _click_reset_stopwatch_button(self):
        """Click the reset stopwatch button"""
        reset_button = self.wait_select_single(
            'ImageButton', objectName='resetButton')
        self.pointing_device.click_object(reset_button)

    @autopilot_logging.log_action(logger.info)
    def create_lap(self):
        """Create a stopwatch lap"""
        self._click_lap_stopwatch_button()

    def _click_lap_stopwatch_button(self):
        """Click the stopwatch lap button"""
        lap_button = self.wait_select_single(
            'ImageButton', objectName='lapButton')
        self.pointing_device.click_object(lap_button)

    @autopilot_logging.log_action(logger.info)
    def delete_lap(self, index):
        """Delete stopwatch lap

        :param index: stopwatch lap number

        """
        lap = self._get_stopwatch_lap_list_element(index=index)
        if not self.main_view.wideAspect:
            self.drag_page_up()
        lap.swipe_to_delete()
        lap.confirm_removal()


class AlarmPage(Page):
    """Autopilot helper for the Alarm page."""

    def get_num_of_alarms(self):
        """Return the number of saved alarms."""
        return int(self._get_saved_alarms_list().count)

    def _get_saved_alarms_list(self):
        """Return the saved alarm list"""
        return self.wait_select_single(
            toolkit_emulators.QQuickListView, objectName='listSavedAlarm')

    @autopilot_logging.log_action(logger.info)
    def add_single_alarm(self, name, day):
        """Add a single type alarm

        :param name: name of alarm
        :param day: day on which the alarm should be triggered

        """
        old_alarm_count = self.get_num_of_alarms()
        add_alarm_page = self._click_add_alarm_button()
        add_alarm_page.set_alarm_name_and_time(name)
        add_alarm_page.set_single_alarm_day(day)
        self._click_save()
        self._wait_to_stop_moving()
        self._confirm_alarm_creation(old_alarm_count)

    @autopilot_logging.log_action(logger.info)
    def add_recurring_alarm(self, name, days):
        """Add a recurring type alarm

        :param name: name of alarm
        :param days: days on which the alarm should be triggered

        """
        old_alarm_count = self.get_num_of_alarms()
        add_alarm_page = self._click_add_alarm_button()
        add_alarm_page.set_alarm_name_and_time(name)
        add_alarm_page.enable_recurring_alarm_type_switch()
        add_alarm_page.set_recurring_alarm_days(days)
        self._click_save()
        self._wait_to_stop_moving()
        self._confirm_alarm_creation(old_alarm_count)

    def _confirm_alarm_creation(self, count):
        """Confirm creation of alarm

        :param count: alarm count before alarm creation

        """
        try:
            self._get_saved_alarms_list().count.wait_for(count + 1)
        except AssertionError:
            raise ClockEmulatorException('Error creating alarm.')

    def _click_add_alarm_button(self):
        """Click the add alarm toolbar button."""
        toolbar = self.main_view.open_toolbar()
        toolbar.click_button('addAlarmButton')
        return self.main_view.wait_select_single(AddAlarmPage)

    def _click_save(self):
        """Click the save alarm button."""
        toolbar = self.main_view.open_toolbar()
        toolbar.click_button('saveAlarmButton')

    @autopilot_logging.log_action(logger.info)
    def delete_alarm(self, index):
        """Delete an alarm at the specified index."""
        old_alarm_count = self.get_num_of_alarms()
        alarm = self.wait_select_single(
            toolkit_emulators.Base, objectName='alarm{}'.format(index))
        if not self.main_view.wideAspect:
            self.drag_page_up()
        alarm.swipe_to_delete()
        alarm.confirm_removal()
        try:
            self._get_saved_alarms_list().count.wait_for(old_alarm_count - 1)
        except AssertionError:
            raise ClockEmulatorException('Error deleting alarm.')

    @autopilot_logging.log_action(logger.info)
    def toggle_alarm(self, index):
        """Toggle an alarm on/off."""
        alarm = self.wait_select_single(
            toolkit_emulators.CheckBox,
            objectName='listAlarmStatus{}'.format(index))
        if not self.main_view.wideAspect:
            self.drag_page_up()
        self.pointing_device.click_object(alarm)

    def get_saved_alarms(self):
        """Return a list with the information of the saved alarms.

        Each item of the returned list is a tuple of
        (name, recurrence, time, enabled).

        """
        alarms = []
        for index in range(self.get_num_of_alarms()):
            name = self.select_single(
                'Label', objectName='listAlarmLabel{}'.format(index)).text
            recurrence = self.select_single(
                'Label', objectName='listAlarmSubtitle{}'.format(index)).text
            # TODO we haven't implemented the alarm time check.
            # --elopio - 2014-02-27
            #time = self.select_single(
            #    'Label', objectName='listAlarmTime{}'.format(index)).text
            enabled = self.select_single(
                toolkit_emulators.CheckBox,
                objectName='listAlarmStatus{}'.format(index)).checked
            alarms.append((name, recurrence, enabled))
        return alarms


class AddAlarmPage(Page):
    """Autopilot helper for the Add Alarm page."""

    @autopilot_logging.log_action(logger.info)
    def set_alarm_name_and_time(self, name):
        """Set alarm name and time

        :param name: name of alarm

        """
        self._set_alarm_time()
        if not self.main_view.wideAspect:
            self.drag_page_up()
        self._set_alarm_name(name)
        self._click_alarm_hour_label()
        if not self.main_view.wideAspect:
            self.drag_page_up()

    def _set_alarm_name(self, name):
        """Set alarm name

        :param name: name of alarm

        """
        name_alarm = self.wait_select_single(
            LabelDots, objectName='addAlarmName')
        # TODO: This wait to ensure that the textfield is visible before
        # entering text should be part of the SDK emulator. Until then, it has
        # been added here. http://pad.lv/1289616  --nik90 2014-03-06
        name_alarm.visible.wait_for(True)
        name_alarm.write(name)

    def _set_alarm_time(self):
        """Set alarm time"""
        self._click_alarm_hour_label()
        hour = self._get_alarm_hour_hand()

        x, y, w, h = hour.globalRect
        tx = x + (w // 2)
        ty = y + (h // 2.5)
        self.pointing_device.drag(
            tx, ty - (h // 4), tx - (w // 2), ty + (h // 2))

        self._click_alarm_minute_label()
        minute = self._get_alarm_minute_hand()

        x, y, w, h = minute.globalRect
        tx = x + (w // 2)
        ty = y + (h // 2.5)
        self.pointing_device.drag(
            tx, ty - (h // 4), tx + (w // 2), ty + (h // 2))

    def _click_alarm_hour_label(self):
        """Click alarm hour label"""
        hour_label = self.wait_select_single('Label', objectName='hourLabel')
        self.pointing_device.click_object(hour_label)

    def _click_alarm_minute_label(self):
        """Click alarm minute label"""
        minute_label = self.wait_select_single(
            'Label', objectName='minuteLabel')
        self.pointing_device.click_object(minute_label)

    def _get_alarm_hour_hand(self):
        """Return the hour hand of clock in add alarm page"""
        return self.wait_select_single('DialerHand', objectName='hourHand')

    def _get_alarm_minute_hand(self):
        """Returns the minute hand of clock in add alarm page"""
        return self.wait_select_single('DialerHand', objectName='minuteHand')

    @autopilot_logging.log_action(logger.info)
    def enable_recurring_alarm_type_switch(self):
        """Enable the recurring alarm type switch"""
        alarm_type_checkbox = self.wait_select_single(
            toolkit_emulators.CheckBox, objectName='RepeatSelector')
        alarm_type_checkbox.check()

    @autopilot_logging.log_action(logger.info)
    def set_single_alarm_day(self, day):
        """Set the alarm day of a single type alarm.

        :param day: single day on which alarm is triggered

        """
        # TODO: we need a proper UITK emulator for optionSelector
        # http://pad.lv/1277059  --nik90 2014-02-06
        day_option_selector = self.select_single(
            'OptionSelector', objectName='OccursSelector')

        # Wait for the option selector to expand
        # FIXME: Replace this when the ui toolkit provides option selector
        # emulator functions. -- nik90 2014-03-06
        if not day_option_selector.currentlyExpanded:
            # Click on days option selector to expand it
            self.pointing_device.click_object(day_option_selector)
            sleep(2)
            day_option_selector.currentlyExpanded.wait_for(True)

        if not self.main_view.wideAspect:
            self.drag_page_up()

        days_object = day_option_selector.select_single(
            'StyledItem', objectName='listContainer')
        dayslist = days_object.select_many('Label', visible=True)

        for index, day_element in enumerate(dayslist):
            if day_element.text == day:
                day_index = index
                break
        else:
            raise ClockEmulatorException('Unknown day: {}'.format(day))

        self.pointing_device.click_object(dayslist[day_index])

    @autopilot_logging.log_action(logger.info)
    def set_recurring_alarm_days(self, days):
        """Set the alarm days of a recurring alarm

        :param days: days on which the alarm is triggered on

        """
        dialog = self._open_recurring_alarm_dialog()
        dialog.set_recurring_alarm_days(days)

    def _open_recurring_alarm_dialog(self):
        """Open the recurring alarm dialogue."""
        recurring_option = self.wait_select_single(
            toolkit_emulators.MultiValue, objectName='OccurrenceList')
        self.pointing_device.click_object(recurring_option)
        alarm_days_dialog = self.main_view.wait_select_single(
            AlarmDays, objectName='alarmDaysDialog')
        alarm_days_dialog.visible.wait_for(True)
        return alarm_days_dialog


class AlarmDays(toolkit_emulators.UbuntuUIToolkitEmulatorBase):
    """Autopilot helper for the Alarm Days dialog."""

    @autopilot_logging.log_action(logger.info)
    def set_recurring_alarm_days(self, days):
        """Set the days in the alarm days dialog

        :param days: days on which the alarm is triggered

        """
        self._fill_recurring_alarm_days(days)
        self._click_confirm_recurring_alarm_days()

    def _fill_recurring_alarm_days(self, days):
        """Choose the alarm days

        :param days: days on which the alarm is triggered

        """
        days_to_select = copy.copy(days)
        for index in range(7):
            day_element_label = self.wait_select_single(
                'Label', objectName='repeatDaysLabel{}'.format(index))
            day_element = self.wait_select_single(
                toolkit_emulators.CheckBox,
                objectName='repeatDaysSwitch{}'.format(index))
            if day_element_label.text in days_to_select:
                day_element.check()
                days_to_select.remove(day_element_label.text)
            else:
                day_element.uncheck()

        if days_to_select != []:
            raise ClockEmulatorException(
                "Can't enable the alarm for days {}".format(days_to_select))

    def _click_confirm_recurring_alarm_days(self):
        """Click the Done button in recurring alarms dialog."""
        done_button = self.wait_select_single(
            'Button', objectName='confirmRepeatDaysButton')
        self.pointing_device.click_object(done_button)


class TimerPage(Page):
    """Autopilot helper for the Timer page."""

    def get_num_of_presets(self):
        """Return the number of presets in the timer page."""
        return int(self.wait_select_single('QQuickListView').count)

    @autopilot_logging.log_action(logger.info)
    def add_timer(self, name, set_time=False):
        """Add a timer preset

        :param name: timer preset name
        :paramx set_time: set timer hour and minute

        """
        self._click_add_timer_button()
        self._wait_to_stop_moving()
        if not self.main_view.wideAspect:
            self.drag_page_up()
        self._set_timer_name(name)
        if not self.main_view.wideAspect:
            self.drag_page_down()
        if set_time:
            self._set_time()
        self._click_save()

    def _click_add_timer_button(self):
        """Click add timer toolbar button"""
        toolbar = self.main_view.open_toolbar()
        toolbar.click_button('addPresetButton')

    def _set_timer_name(self, name):
        """Set timer preset name

        :param name: timer preset name

        """
        name_label = self.wait_select_single(
            LabelDots, objectName='namePreset')
        # TODO: This wait to ensure that the textfield is visible before
        # entering text should be part of the SDK emulator. Until then, it has
        # been added here. http://pad.lv/1289616  --nik90 2014-03-06
        name_label.visible.wait_for(True)
        name_label.write(name)
        if name_label.text != name:
            raise ClockEmulatorException('Failed to write the timer name.')

    def _set_time(self):
        """Set the timer hour and minute"""
        # Set timer minute
        logger.debug('Grabbing minute hand')

        timeout = 0
        timer_face = self.select_single('TimerFace')
        #need to select clock in order to set time
        while not timer_face.focus and timeout < 10:
            logger.debug('Timerlabel loop')
            self._click_timer_label()
            sleep(1)
            timeout += 1

        # TODO we need a dialer hand emulator, with proper self-tests.
        # --elopio -2014-01-31
        minute = self._get_timer_minute_hand()
        #make sure timer is set
        x, y, w, h = minute.globalRect
        tx = x + (w / 2)
        ty = y + (h / 2.5)
        timeout = 0
        reset_button = self._get_timer_reset_button()
        logger.debug('reset_button %s' % reset_button.visible)

        #Try again if we failed at first
        while not reset_button.visible and timeout < 10:
            logger.debug('Dragging minute hand')
            self.pointing_device.drag(
                tx, ty - (h / 4), tx + (w / 2), ty + (h / 2))
            sleep(1)
            timeout += 1

    def _get_timer_minute_hand(self):
        """Return the hour hand of clock in timer tab."""
        return self.wait_select_single('DialerHand', objectName='minuteHand')

    def _click_timer_label(self):
        timer_label = self._get_label_timer()
        self.pointing_device.click_object(timer_label)

    def _get_label_timer(self):
        """Return the label with the timer countdown."""
        return self.select_single('Label', objectName='label')

    def _get_timer_reset_button(self):
        """Return the select for the reset button."""
        return self.select_single('ImageButton', objectName='resetButtonTimer')

    def _click_save(self):
        """Click the save timer toolbar button"""
        toolbar = self.main_view.open_toolbar()
        toolbar.click_button('savePresetButton')

    def get_presets(self):
        """Return a list with the information of the presets.

        Each item of the returned list is a tuple of (name, time).

        """
        presets = []
        for index in range(self.get_num_of_presets()):
            text = self.select_single(
                'Label', objectName='presetTextLabel{}'.format(index)).text
            time = self.select_single(
                'Label', objectName='presetTimerLabel{}'.format(index)).text
            presets.append((text, time))
        return presets

    @autopilot_logging.log_action(logger.info)
    def run_preset_timer(self, index):
        """Run timer preset

        :param index: index of timer preset to be run

        """
        if not self.main_view.wideAspect:
            self.drag_page_up()
        self._select_preset(index)
        self._click_timer_label()

    def _select_preset(self, index):
        """Return the timer preset object

        :param index: index of timer preset to be selected

        """
        presets_list = self.wait_select_single(
            toolkit_emulators.QQuickListView, objectName='listTimerPreset')
        presets_list.click_element('preset{}'.format(index))

    def get_current_time(self):
        """Return the time of the currently selected timer."""
        self._get_label_timer().text

    @autopilot_logging.log_action(logger.info)
    def edit_preset_timer(self, name, index):
        """Edit timer preset

        :param name: name of timer preset
        :param index: index of the timer preset to be edited

        """
        if not self.main_view.wideAspect:
            self.drag_page_up()
        self._select_preset(index)
        self._set_timer_name(name)
        self._click_save()

    @autopilot_logging.log_action(logger.info)
    def delete_preset_timer(self, index):
        """Delete timer preset

        :param index: timer preset index

        """
        if not self.main_view.wideAspect:
            self.drag_page_up()
        preset = self.select_single(
            toolkit_emulators.Standard, objectName='preset{}'.format(index))
        preset.swipe_to_delete()
        preset.confirm_removal()
