# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013, 2014 Canonical Ltd
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authored by: Nicholas Skaggs <nicholas.skaggs@canonical.com>
#              Nekhelesh Ramananthan <krnekhelesh@gmail.com>

"""Tests for the Clock App - Alarm"""

from __future__ import absolute_import

import datetime
import unittest

from autopilot.matchers import Eventually
from testtools.matchers import Equals
from ubuntu_clock_app.tests import ClockAppTestCase


class TestAlarm(ClockAppTestCase):
    """Tests the alarm page features"""

    def setUp(self):
        """ This is needed to wait for the application to start.

        In the testfarm, the application may take some time to show up.

        """
        super(TestAlarm, self).setUp()
        self.assertThat(
            self.main_view.visible, Eventually(Equals(True)))

        self.page = self.main_view.open_alarm()

    def test_add_single_type_alarm_must_add_to_alarm_list(self):
        """Test to check if a single type alarm is saved properly

        This test saves a single type alarm and verifies if it is added to the
        alarm list in the alarm page.

        """
        # The alarm day is always set to tomorrow to ensure that it is
        # not rejected by the clock app since the clock app does not
        # allow alarms to be created before the current time of the same
        # day. This limitation will be removed later.
        # -- nik90 2014-02-06
        tomorrow = datetime.datetime.now() + datetime.timedelta(days=1)
        test_alarm_name = 'Single Test'
        # TODO this will be affected by the locale. --elopio - 2014-02-27
        expected_recurrence = 'Once on {}'.format(tomorrow.strftime('%a'))
        expected_enabled_value = True
        expected_alarm_info = (
            test_alarm_name, expected_recurrence, expected_enabled_value)

        tomorrow_day = tomorrow.strftime('%A')
        self.page.add_single_alarm(test_alarm_name, tomorrow_day)

        saved_alarms = self.page.get_saved_alarms()
        # TODO we are not checking the alarm time. --elopio - 2014-02-27
        self.assertIn(expected_alarm_info, saved_alarms)

        # TODO: Remove this statement once proper support for cleaning the
        # test alarm environment is added. Until then remove the alarm
        # created during the test at the end of the test.
        # -- nik90 - 2014-03-03
        self.page.delete_alarm(index=0)

    def test_delete_alarm_must_delete_from_alarm_list(self):
        """Test to delete an alarm and check if it has been properly removed"""
        # FIXME: Instead of always deleting the first alarm, run the alarm
        # which was created. To do this in a nice way, we should use the
        # QQuickListView but it has a bug that makes it fail in this case:
        # http://pad.lv/1275060 --elopio - 2014-01-31

        # (set-up) Create a recurring alarm
        self.page.add_recurring_alarm(
            'Test alarm to delete', ['Monday', 'Tuesday'])

        old_alarm_count = self.page.get_num_of_alarms()
        self.page.delete_alarm(index=0)

        # Check that the alarm is deleted
        self.assertThat(self.page.get_num_of_alarms,
                        Eventually(Equals(old_alarm_count - 1)))

    @unittest.skip("Pending development, bug 1272337")
    def test_toggle_alarm_status_must_enable_or_disable_alarm(self):
        """Test to toggle an alarm and check it has been done properly"""
        # FIXME: This test is skipped due to an upstream bug in EDS which
        # causes the alarm status to not be properly toggled. This could
        # result in flaky test.
        # http://pad.lv/1272337 -- nik90 - 2014-02-07

        # (set-up) Create a recurring alarm
        self.page.add_recurring_alarm(
            'Test alarm to delete', ['Monday', 'Tuesday'])

        self.page.toggle_alarm(index=0)


class TestRecurringAlarm(ClockAppTestCase):
    """Tests the recurring alarm creation"""

    scenarios = [
            ('weekday',
            {'alarm_name': 'Weekday Alarm Test',
             'days': ['Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday'],
             'expected_recurrence': 'Every weekday'
            }),

            ('weekend',
            {'alarm_name': 'Weekend Alarm Test',
             'days': ['Saturday', 'Sunday'],
             'expected_recurrence': 'Every Sat,Sun'
            }),

            ('daily',
            {'alarm_name': 'Daily Alarm Test',
             'days': ['Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday',
                      'Saturday', 'Sunday'],
             'expected_recurrence': 'Daily'
            })
        ]

    def setUp(self):
        """ This is needed to wait for the application to start.

        In the testfarm, the application may take some time to show up.

        """
        super(TestRecurringAlarm, self).setUp()
        self.assertThat(
            self.main_view.visible, Eventually(Equals(True)))

        self.page = self.main_view.open_alarm()

    def test_add_recurring_type_alarm_must_add_to_alarm_list(self):
        """ Test to check if a recurring alarm is saved properly

        Recurring alarms are triggered on more than one day. This test saves a
        recurring alarm and check if it has been saved and is added to the
        alarm list in the alarm page.

        """
        # XXX this will be affected by the locale. --elopio - 2014-02-27
        expected_alarm_status = True
        expected_alarm_info = (
            self.alarm_name, self.expected_recurrence, expected_alarm_status)

        self.page.add_recurring_alarm(self.alarm_name, self.days)

        saved_alarms = self.page.get_saved_alarms()
        # TODO we are not checking the alarm time. --elopio - 2014-02-27
        self.assertIn(expected_alarm_info, saved_alarms)

        # TODO: Remove this statement once proper support for cleaning the
        # test alarm environment is added. Until then remove the alarm
        # created during the test at the end of the test.
        # -- nik90 - 2014-03-03
        self.page.delete_alarm(index=0)