/*
 * Copyright (C) 2013, 2014 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Alessandro Pozzi <signor.hyde@gmail.com>
 *              Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 */

import QtQuick 2.0
import U1db 1.0 as U1db
import Ubuntu.Layouts 0.1
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import Ubuntu.Components.Themes.Ambiance 0.1
import "../common/ClockUtils.js" as Utils
import "../common"

Page {
    id: timerPage

    property Flickable timerPageFlickable

    readonly property real timerHeaderHeight: units.gu(9.5)
    property alias minutes: analogTimer.minutes;
    property alias seconds: analogTimer.seconds;
    property alias totalTime: analogTimer.totalTime;
    property alias timerOn: analogTimer.timerOn;
    property bool setFocus: false
    property bool isRunning: analogTimer.state !== "" || timerPage.state !== "" || analogTimer.minutes !== 0 || analogTimer.seconds !== 0

    flickable: !wideAspect ? timerPageFlickable : null

    // Default Timer Presets as defined by the design team
    U1db.Document {
        database: db
        docId: "defaultTimerOne"
        create: true
        defaults: { "timer": { "name": i18n.tr("Soft-boiled egg"), "duration": 180 }}
    }

    U1db.Document {
        database: db
        docId: "defaultTimerTwo"
        create: true
        defaults: { "timer": { "name": i18n.tr("Aeropress coffee"), "duration": 165 }}
    }

    U1db.Document {
        database: db
        docId: "defaultTimerThree"
        create: true
        defaults: { "timer": { "name": i18n.tr("Al dente"), "duration": 600 }}
    }

    U1db.Index {
        database: db
        id: by_timer
        expression: ["timer.name", "timer.duration"]
    }

    U1db.Query {
        id: timers
        index: by_timer
        query: ["*", "*"]
    }

    // Function to add a timer preset
    function add_preset() {
        reset()
        timerPage.state = "addPreset"
        if(timerPage.setFocus)
            namePreset.forceActiveFocus()
    }

    // Function to cancel a timer preset
    function cancel_preset() {
        if(timerPage.state === "addPreset") {
            reset()
            timerPage.state = "";
            timerPage.setFocus = false
            toolbarTimer.opened = false
        }
    }

    // Function to save timer presets to u1db
    function savePreset(label, seconds, index, docId) {
        if(index === -1) {
            Utils.log("Saving a new preset")
            db.putDoc({ "timer": { "name": label, "duration": seconds }})
        }
        else {
            Utils.log("Updating an existing preset")
            db.putDoc({ "timer": { "name": label, "duration": seconds }}, listPreset.listDocId)
        }
    }

    actions:
        Action {
        id: addTimerAction
        iconSource: Qt.resolvedUrl("../images/add_icon.png")
        text: i18n.tr("Add Timer")
        description: i18n.tr("Add a new timer")
        // TRANSLATORS: this is a list of semicolon-separated keywords to be used to activate this action (Add Timer) in the HUD
        keywords: i18n.tr("Add;Timer;Eggtimer;Preset;Ticker")
        onTriggered: add_preset()
    }

    states: [
        State {
            name: "selectPreset"
            when: listPreset.listIndex != -1
            PropertyChanges { target: namePreset; visible: true }
            PropertyChanges { target: editPresetToolbarButton; visible: true }
            PropertyChanges { target: addPresetToolbarButton; visible: false }
        },
        State {
            name: "addPreset"
            PropertyChanges { target: analogTimer; enableMouseArea: false }
            PropertyChanges { target: listPreset; visible: false }
            PropertyChanges { target: namePreset; visible: true }
            // TODO: Fix autopilot test first before enabling this change in another MP
            //PropertyChanges { target: namePreset; text: "Timer #" + (listPreset.listCount + 1) }
            PropertyChanges { target: addPresetToolbarButton; visible: false }
            PropertyChanges { target: editPresetToolbarButton; visible: true }
            PropertyChanges { target: cancelPresetToolbarButton; visible: true }
        }
    ]

    function onTimerUpdate() {
        analogTimer.onTimerUpdate();
    }

    // Function to print timer label (format mm:ss and hh:mm:ss when hh > 0) at top
    function setTimerLabel(minutes, seconds) {
        if (minutes > 59) { // Do not show "60" value for minutes
            minutes = 0
        }
        if (seconds > 59) { // Do not show "60" value for seconds
            seconds = 0
        }
        analogTimer.minutes = Math.round(minutes) // This makes minute hand stick to digits
        analogTimer.seconds = Math.round(seconds) // This makes second hand stick to digits
        return Utils.zeroleft(minutes, 1) + ":" + Utils.zeroleft(seconds, 2);
    }

    function reset() {
        analogTimer.reset();
        listPreset.listIndex = -1;
        namePreset.text = "";
    }

    Component.onCompleted: Utils.log("TimerPage loaded");
    Component.onDestruction: Utils.log("TimerPage unloaded")

    Layouts {
        anchors.fill: parent
        layouts: [
            // Tablet/Desktop Layout rules
            ConditionalLayout {
                name: "tablet"
                when: wideAspect
                Row {
                    anchors.fill: parent
                    Item {
                        id: timerFaceTabletLayoutContainer
                        width: listPreset.listCount !== 0 && timerPage.state !== "addPreset" ? timerPage.width - sidebar.width : timerPage.width

                        Behavior on width {
                            UbuntuNumberAnimation {}
                        }

                        anchors {
                            top: parent.top
                            bottom: parent.bottom
                        }

                        ItemLayout {
                            id: timerFaceTabletItem
                            item: "timerFace"
                            width: analogTimer.width
                            height: analogTimer.height
                            anchors {
                                centerIn: parent
                                verticalCenterOffset: - namePresetTabletItem.height - units.gu(2)
                            }
                        }

                        ItemLayout {
                            id: namePresetTabletItem
                            item: "namePreset"
                            width: namePreset.width
                            height: namePreset.contentHeight
                            anchors {
                                horizontalCenter: parent.horizontalCenter
                                top: timerFaceTabletItem.bottom
                                topMargin: units.gu(2)
                            }
                        }

                        ItemLayout {
                            id: resetButtonTimerTabletItem
                            item: "resetButtonTimer"
                            width: resetButtonTimer.width
                            height: resetButtonTimer.height
                            anchors {
                                horizontalCenter: parent.horizontalCenter
                                top: namePresetTabletItem.bottom
                                topMargin: units.gu(2)
                            }
                        }
                    }

                    // Sidebar to hold the timer preset column in the tablet layout
                    Sidebar {
                        id: sidebar
                        mode: "right"
                        visible: listPreset.listCount !== 0 && timerPage.state !== "addPreset"

                        anchors {
                            top: parent.top
                            bottom: parent.bottom
                        }

                        ItemLayout {
                            item: "listPreset"
                            width: parent.width
                        }
                    }
                }
            }
        ]

        Flickable {
            id: timerAnimationContainer
            objectName: "animationContainer"

            Component.onCompleted: timerPageFlickable = timerAnimationContainer

            anchors.fill: parent
            contentWidth: timerPage.width
            contentHeight: timerFacePhoneLayoutContainer.height + listPreset.height

            // Container to hold the timer face and timer label
            Item {
                id: timerFacePhoneLayoutContainer

                width: parent.width
                height: timerPage.height - timerPage.timerHeaderHeight - listPreset.headerHeight - listPreset.dividerHeight

                // Main element which draws the timer face and manages time.
                TimerFace {
                    id: analogTimer
                    Layouts.item: "timerFace"

                    innerLabel.text: setTimerLabel(minutes, seconds);

                    anchors {
                        centerIn: parent
                        verticalCenterOffset: - namePreset.height
                    }

                    Timer {
                        id: seconds_timer
                        interval: 250
                        repeat: true
                        onTriggered: analogTimer.innerLabel.text = setTimerLabel(minutes, seconds);
                    }

                    inProgressFlag: (timerOn === true || analogTimer.state === "DONE") ? true : false;
                    onMinutesChanged: innerLabel.text = setTimerLabel(minutes, seconds);
                    onSecondsChanged: if (timerOn == true) seconds_timer.restart()
                                      else innerLabel.text = setTimerLabel(minutes, seconds);

                    states:[
                        State { name: "STOP" },
                        State { name: "PAUSE"},
                        State {
                            name: "DONE"
                            when: (totalTime == -1 && timerOn == false)
                            PropertyChanges { target: analogTimer; timerHintImage: Qt.resolvedUrl("../images/timer_green.png") }
                        }
                    ]

                    onClicked: {
                        switch (analogTimer.state) {

                        case "":
                            if ((seconds != 0 || minutes != 0) || timerOn) {
                                totalTime = (Math.round(minutes) * 60) + seconds;
                                analogTimer.startTime = new Date();
                                timerOn = true;
                                analogTimer.state = "STOP";
                            }
                            break;

                        case "STOP":
                            timerOn = false;
                            analogTimer.pauseTime = analogTimer.totalTime - analogTimer.remTime + 1; // +1 is necessary to start immediately the timer when finish a pause
                            analogTimer.state = "PAUSE";
                            break;

                        case "DONE":
                            timerPage.reset();
                            analogTimer.state = "";
                            break;

                        case "PAUSE":
                            analogTimer.startTime = new Date();
                            timerOn = true;
                            analogTimer.state = "STOP";
                            break;
                        }
                    }
                }

                // Element to set the name of a saved preset.
                LabelDots {
                    id: namePreset
                    objectName: "namePreset"
                    Layouts.item: "namePreset"

                    visible: false
                    width: analogTimer.width;
                    height: namePreset.contentHeight
                    anchors {
                        horizontalCenter: parent.horizontalCenter
                        top: analogTimer.bottom
                        topMargin: units.gu(2)
                    }
                    placeholderText: i18n.tr("Preset Name")
                    onClickedOutside: analogTimer.forceActiveFocus()
                }

                // Reset button to reset timer
                ImageButton {
                    id: resetButtonTimer
                    objectName: "resetButtonTimer"
                    Layouts.item: "resetButtonTimer"

                    anchors {
                        horizontalCenter: parent.horizontalCenter
                        top: namePreset.bottom
                        topMargin: units.gu(2)
                    }

                    buttonImage: Qt.resolvedUrl("../images/reset_icon.svg");
                    buttonLabel: i18n.tr("Reset");
                    visible: timerOn || (minutes != 0 || seconds != 0)? true : false;
                    enabled: true
                    onClicked: reset();
                }
            }

            // Column element to hold the saved presets
            PresetList {
                id: listPreset
                Layouts.item: "listPreset"
                model: timers
                height: childrenRect.height
                width: parent.width
                anchors.top: timerFacePhoneLayoutContainer.bottom
                maxItemsVisible: !wideAspect ? 5 : parseInt((timerPage.height - headerHeight)/units.gu(6))
            }
        }
    }

    // Various toolbar actions such as add preset, edit and back.
    tools: ToolbarItems {
        id: toolbarTimer

        locked: false
        opened: false

        onLockedChanged: opened = locked

        ToolbarButton {
            id: editPresetToolbarButton
            objectName: "savePresetButton"
            visible: false
            iconSource: Qt.resolvedUrl("../images/select_icon.png")
            text: i18n.tr("Save")
            onTriggered: {
                savePreset(namePreset.text, ((minutes * 60) + seconds), listPreset.listIndex, listPreset.listDocId)
                namePreset.focus = false
                if (listPreset.listIndex == -1) {
                    reset()
                    timerPage.state = "";
                }
                timerPage.setFocus = false
                toolbarTimer.opened = false
            }
        }

        ToolbarButton {
            id: addPresetToolbarButton
            objectName: "addPresetButton"
            action: addTimerAction
        }

        ToolbarButton {
            id: cancelPresetToolbarButton
            objectName: "cancelPresetButton"
            visible: false;
            iconSource: Qt.resolvedUrl("../images/cancel_icon.svg")
            text: i18n.tr("Cancel")
            onTriggered: cancel_preset()
        }
    }
}
