/*
 * Copyright (C) 2013, 2014 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Juha Ristolainen <juha.ristolainen@codemancers.fi>
 *              Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 *              Nick Leppänen Larsson <frals@frals.se>
 *              Marco Biscaro <marcobiscaro2112@gmail.com>
 *              Alessandro Pozzi <signor.hyde@gmail.com>
 */

import QtQuick 2.0
import U1db 1.0 as U1db
import Ubuntu.Components 0.1
import "common/ClockUtils.js" as Utils
import "common/KeyboardShortcuts.js" as Keyboard
import "clock"
import "alarm"
import "stopwatch"
import "timer"

MainView {
    id: root
    objectName: "clock"

    applicationName: "com.ubuntu.clock"

    width: units.gu(50)
    height: units.gu(80)

    // StateSaver properties
    StateSaver.properties: "width, height"

    headerColor: "#57365E"
    backgroundColor: "#A55263"
    footerColor: "#D75669"

    focus: true
    Keys.onReturnPressed: Keyboard.enterShortcut(event, rootTabs.selectedTabIndex, stopwatchLoader.item)
    Keys.onEnterPressed: Keyboard.enterShortcut(event, rootTabs.selectedTabIndex, stopwatchLoader.item)
    Keys.onSpacePressed: Keyboard.spaceShortcut(event, rootTabs.selectedTabIndex, stopwatchLoader.item)
    Keys.onEscapePressed: Keyboard.escapeShortcut(event, rootTabs.selectedTabIndex, pagestack, timerLoader.item, stopwatchLoader.item)
    Keys.onPressed: rootTabs.selectedTabIndex = Keyboard.generalShortcuts(event, rootTabs.count, rootTabs.selectedTabIndex, pagestack, timerLoader.item, stopwatchLoader.item)

    // Property to store the default application dimensions
    readonly property real minimumWidth: units.gu(50)
    readonly property real minimumHeight: units.gu(75)

    // Property to determine whether to morph to tablet interface
    property bool wideAspect: root.width > units.gu(80)

    // Property to store the state of an application (active or suspended)
    property bool applicationState: Qt.application.active

    // Property to store the current time
    property var now

    // Initialization after page has loaded
    Component.onCompleted: {
        // Any init here
        Utils.log("MainView loaded");
        // When root component is completed the StateSaver restore it to previous dimensions.
        // If it is too small, set height and width to a proper size.
        if((root.width < minimumWidth) || (root.height < minimumHeight)){
            root.width = minimumWidth
            root.height = minimumHeight
        }
    }

    actions: [
        Action {
            id: appSettingsAction
            text: i18n.tr("Settings")
            // TRANSLATORS: this is a list of semicolon-separated keywords to be used to activate this action (Settings) in the HUD
            keywords: i18n.tr("Settings;Setting;Configuration;")
            description: i18n.tr("Application Settings")
            iconSource: Qt.resolvedUrl("images/settings.svg")
            onTriggered: pagestack.push(Qt.resolvedUrl("common/SettingsPage.qml"))
        }
    ]

    // Timer that runs our app
    Timer {
        id: updateTimer
        objectName: "updateTimer"

        interval: 1000
        repeat: true
        running: true

        onTriggered: {
            now = new Date();
            if(clockLoader.source != "")
                clockLoader.item.onTimerUpdate(now);
            if(alarmLoader.source != "")
                alarmLoader.item.onTimerUpdate(now);
            if(timerLoader.source != "")
                timerLoader.item.onTimerUpdate(now);
        }
    }

    // Database to store app data locally
    U1db.Database {
        id: db
        path: "settings"
    }

    // Document to store app settings data
    U1db.Document {
        id: appSetting
        database: db
        docId: "appSettings"
        create: true
        defaults: { "timeFormat": "12-hour", "style": "analogue", "seconds": "false" }
    }

    ////////////////////////////////////////////////////////////////////////////////
    // UI
    PageStack {
        id: pagestack

        Component.onCompleted: push(rootTabs);

        // Applications pages in tabs (Clock, Alarm, Timer, Stopwatch)
        Tabs {
            id: rootTabs
            objectName: "rootTabs"
            anchors.fill: parent

            onSelectedTabChanged: {
                if(rootTabs.selectedTab === stopwatchTab)
                    stopwatchLoader.source = Qt.resolvedUrl("stopwatch/StopwatchPage.qml")
                else if(stopwatchLoader.source != "" && !stopwatchLoader.item.isRunning)
                        stopwatchLoader.source = ""

                if(rootTabs.selectedTab === timerTab)
                    timerLoader.source = Qt.resolvedUrl("timer/TimerPage.qml")
                else if(timerLoader.source != "" && !timerLoader.item.isRunning)
                    timerLoader.source = ""

                if(rootTabs.selectedTab === alarmTab)
                    alarmLoader.source = Qt.resolvedUrl("alarm/AlarmPage.qml")
                else
                    alarmLoader.source = ""

                if(rootTabs.selectedTab === clockTab)
                    clockLoader.source = Qt.resolvedUrl("clock/ClockPage.qml")
                else
                    clockLoader.source = ""
            }

            Tab {
                id: clockTab
                objectName: "ClockTab"

                title: i18n.tr("Clock")

                // Tab content begins here
                page: Loader {
                    id: clockLoader
                    parent: clockTab
                    anchors {
                        left: parent.left
                        right: parent.right
                        bottom: parent.bottom
                    }
                }
            }

            // Disabled Alarm Tab until the feature is ready for use.
            Tab {
                id: alarmTab
                objectName: "AlarmTab"

                title: i18n.tr("Alarm")

                AlarmModel {
                    id: alarmModel
                    Component.onCompleted: Utils.log("Alarm Database loaded")
                    onModelReset: if (alarmLoader.source != "") alarmLoader.item.get_next_active_alarm()
                }

                page: Loader {
                    id: alarmLoader
                    parent: alarmTab
                    anchors {
                        left: parent.left
                        right: parent.right
                        bottom: parent.bottom
                    }
                }
            }

            Tab {
                id: timerTab
                objectName: "TimerTab"

                title: i18n.tr("Timer")

                page: Loader {
                    id: timerLoader
                    parent: timerTab
                    anchors {
                        left: parent.left
                        right: parent.right
                        bottom: parent.bottom
                    }
                }
            }

            Tab {
                id: stopwatchTab
                objectName: "StopwatchTab"

                title: i18n.tr("Stopwatch")

                page: Loader {
                    id: stopwatchLoader
                    parent: stopwatchTab
                    anchors {
                        left: parent.left
                        right: parent.right
                        bottom: parent.bottom
                    }
                }
            }
        }
    }
}
